/*
  crossword -- a crossword game
  Copyright (C) 2000 Falk Hueffner

  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or (at your option)
  any later version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 59 Temple
  Place, Suite 330, Boston, MA 02111-1307 USA

  $Id: WordListTrie.cc,v 1.4 2000/12/05 14:54:01 falk Exp $
*/

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>

#include "WordListTrie.hh"

WordListTrie::WordListTrie(std::istream& nin) : in(nin), _done(false) {
    _numReservedNodes = 65536;
    nodes.resize(1);
    _numNodes = 0;
    nodes[_numNodes++] = TrieNode('*', false, 1, true);

    readNextWord();
    //nodes[0]._firstChildOffset = buildNode(0); // WTF does this not work???
    int firstRootChildOffset = buildNode(0);
    nodes[0]._firstChildOffset = firstRootChildOffset;

    // relativate child indices
    for (int i = 0; i < _numNodes; ++i) {
	if (nodes[i]._firstChildOffset != 0)
	    nodes[i]._firstChildOffset -= i;
    }
}

int WordListTrie::buildNode(unsigned int depth) { // returns index of first child
    assert(!_done);
    if (_currentWord.length() == depth) {
	// End of word reached. If the next word isn't a continuation of
	// the current one, then we've reached the bottom of the recursion
	// tree.
	readNextWord();
	if (_firstDiff < depth)
	    return 0;
    }

    std::vector<TrieNode> edges;	// FIXME find good name

    // Loop through all the sub-nodes until a word is read which can't be
    // reached via this node.
    do {
	char letter = _currentWord[depth];
	bool isTerminal = _currentWord.length() - 1 == depth;
	edges.push_back(TrieNode(letter, isTerminal, buildNode(depth + 1)));
    } while (_firstDiff == depth && !_done);

    if (!_done)
	assert(_firstDiff < depth);

    edges.back()._isLastSibling = true;

    return addNodes(edges);
}

int WordListTrie::addNodes(const std::vector<TrieNode>& edges) {
    NodeCache::const_iterator n = nodeCache.find(edges);
    if (n != nodeCache.end())
	return n->second;

    nodes.resize(nodes.size() + edges.size());

    int firstFreeIndex = _numNodes;
    for (unsigned int i = 0; i < edges.size(); ++i)
	nodes[_numNodes + i] = edges[i];
	
    _numNodes += edges.size();

    nodeCache[edges] = firstFreeIndex;

    return firstFreeIndex;
}

void WordListTrie::readNextWord() {
    std::string nextWord;
    do {
	getline(in, nextWord);
	if (!in) {
	    _done = true;

	    return;
	}
    } while (nextWord.length() == 0); // skip blank lines
    assert(nextWord.length() > 1);
    assert(nextWord > _currentWord);
    int numCommonLetters = 0;
    int l = std::min(_currentWord.length(), nextWord.length());
    while (_currentWord[numCommonLetters] == nextWord[numCommonLetters]
	   && numCommonLetters < l)
	++numCommonLetters;
    _firstDiff = numCommonLetters;
    _currentWord = nextWord;
}

void WordListTrie::dump(std::ostream& out) {
    out.write((char*) &_numNodes, sizeof(int));
    out.write((char*) &nodes[0], _numNodes * sizeof(TrieNode));
}
